;;; This file is part of guile-game, a graphic library for Guile

;;; Copyright (C) 2010  Luca Saiu

;;; This program is free software: you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation, either version 3 of the License, or
;;; (at your option) any later version.

;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.

;;; You should have received a copy of the GNU General Public License
;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.

;; The files in this directory are the actual Scheme sources of my
;; implementation, which my students couldn't originally access; their
;; task consisted in implementing similar funcionality, so I made it
;; (well, let's say relatively) hard to look at what I did.
;;   Everything is realeased under the GPLv3-or-later at this point,
;; if you are reading this.


;;;; My alternate alist functions:
(define (unbind-function k a)
  (cond ((null? a) a)
        ((eq? (caar a) k) (cdr a))
        (else (cons (car a)
                    (unbind-function k (cdr a))))))
(define-macro (unbind k a) `(unbind-function ',k ,a))
(define (bind-function k v a)
  (acons k v (unbind-function k a)))
(define-macro (bind1 k v a) `(acons ',k ,v (unbind ,k ,a)))
(define-macro (bind . bindings-and-a)
  (if (null? (cdr bindings-and-a))
      (car bindings-and-a)
      `(bind1 ,(car bindings-and-a)
              ,(cadr bindings-and-a)
              (bind ,@(cddr bindings-and-a)))))
(define (lookup-function k a)
  (cond ((null? a)
         (error "lookup-function: unbound key" k))
        ((eq? (caar a) k)
         (cdar a))
        (else
         (lookup-function k (cdr a)))))
(define-macro (lookup k a)
  `(lookup-function ',k ,a))
(define (bound-function? k a)
  (cond ((null? a)
         #f)
        ((eq? (caar a) k)
         #t)
        (else
         (bound-function? k (cdr a)))))
(define-macro (bound? k a)
  `(bound-function? ',k ,a))

(define (is-in? x list)
  (cond ((null? list)
         #f)
        ((eq? x (car list))
         #t)
        (else
         (is-in? x (cdr list)))))

(define (square x)
  (* x x))

(define (distance player ball)
  (let ((player-x (lookup x player))
        (player-y (lookup y player))
        (ball-x (lookup x ball))
        (ball-y (lookup y ball)))
    ;; Ignore the x coordinates: we're only interested in
    ;; "near" objects according to the y coordinate:
    (abs (- ball-y player-y))))

;;    (sqrt (+ (square (- ball-x player-x))
;;             (square (- ball-y player-y))))))

(define (nearest-ball-with-accumulator player other-balls best-distance best-ball)
  (if (null? other-balls)
      best-ball
      (let* ((new-ball (car other-balls))
             (new-distance (distance player new-ball)))
        (if (>= new-distance best-distance)
            (nearest-ball-with-accumulator player (cdr other-balls) best-distance best-ball)
            (nearest-ball-with-accumulator player (cdr other-balls) new-distance new-ball)))))

(define (nearest-ball player balls)
  (nearest-ball-with-accumulator player
                                 (cdr balls)
                                 (distance player (car balls))
                                 (car balls)))

(define (decide-command state)
  (let* ((our-id (lookup your-id state))
         (current-players (map car (lookup players state)))
         (shall-we-join (not (is-in? our-id current-players))))
    (if shall-we-join
        '(join)
        (let* ((our-player (lookup-function our-id (lookup players state)))
               (our-x (lookup x our-player))
               (balls (lookup balls state))
               (relevant-ball (nearest-ball our-player balls))
               (ball-x (lookup x relevant-ball)))
          (if (< ball-x our-x)
              '(left)
              '(right))))))

(while #t
  (let ((state (read)))
    (if (eof-object? state)
        (exit -1)
        (let ((command (decide-command state)))
          (write command)
          (newline)
          (force-output)))))
