;;; This file is part of guile-with-graphics, a graphic library for Guile

;;; Copyright (C) 2009  Luca Saiu
;;; Updated in 2010 by Luca Saiu

;;; This program is free software: you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation, either version 3 of the License, or
;;; (at your option) any later version.

;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.

;;; You should have received a copy of the GNU General Public License
;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.

;;; Support for different levels of detail. The best-looking levels are
;;; computation-intensive, so this is useful for slow computers:
(define draw-possibly-nuanced-circle draw-nuanced-circle)
(define detail-levels
  '(nuanced circle circle-border square square-border))
(define detail-level
  (car detail-levels))
(define (rotate-detail-level!)
  (set! detail-levels (append (cdr detail-levels)
                              (list (car detail-levels))))
  (set! detail-level (car detail-levels))
  (case detail-level
    ((nuanced)
     (set! draw-possibly-nuanced-circle draw-nuanced-circle))
    ((circle)
     (set! draw-possibly-nuanced-circle draw-circle))
    ((circle-border)
     (set! draw-possibly-nuanced-circle draw-circle-border))
    ((square)
     (set! draw-possibly-nuanced-circle draw-centered-square))
    ((square-border)
     (set! draw-possibly-nuanced-circle draw-centered-square-border))
    (else
     (error "Unknown detail level" detail-level))))

;;; Draw pills (each pill is simply represented by the cons of its two coordinates):
(define pill-color (make-color .5 1 .5))
(define (draw-pill pill)
  (draw-possibly-nuanced-circle (pill-x pill) (pill-y pill) pill-radius pill-color))

;; ;;; Draw the player:
;; (define left-eye-delta-angle 0.5)
;; (define right-eye-delta-angle (- left-eye-delta-angle))
;; (define eye-color (light (light yellow)))
;; (define avatar-name-color (make-color 1 1 1))
;; (define (draw-player x y radius angle name color)
;;   (let* ((left-eye-angle (+ angle left-eye-delta-angle))
;;          (right-eye-angle (+ angle right-eye-delta-angle))
;;          (tail-angle (+ angle pi))
;;          (sin-tail-angle (sin tail-angle))
;;          (cos-tail-angle (cos tail-angle))
;;          (tail-segment-1-distance (* radius 1.0))
;;          (tail-segment-2-distance (* radius 1.3))
;;          (tail-segment-3-distance (* radius 1.5))
;;          (tail-segment-4-distance (* radius 1.7))
;;          (eye-radius (/ radius 3))
;;          (left-eye-center-x (+ x (* radius (cos left-eye-angle))))
;;          (left-eye-center-y (- y (* radius (sin left-eye-angle))))
;;          (right-eye-center-x (+ x (* radius (cos right-eye-angle))))
;;          (right-eye-center-y (- y (* radius (sin right-eye-angle))))
;;          (tail-segment-1-center-x (+ x (* tail-segment-1-distance cos-tail-angle)))
;;          (tail-segment-1-center-y (- y (* tail-segment-1-distance sin-tail-angle)))
;;          (tail-segment-2-center-x (+ x (* tail-segment-2-distance cos-tail-angle)))
;;          (tail-segment-2-center-y (- y (* tail-segment-2-distance sin-tail-angle)))
;;          (tail-segment-3-center-x (+ x (* tail-segment-3-distance cos-tail-angle)))
;;          (tail-segment-3-center-y (- y (* tail-segment-3-distance sin-tail-angle)))
;;          (tail-segment-4-center-x (+ x (* tail-segment-4-distance cos-tail-angle)))
;;          (tail-segment-4-center-y (- y (* tail-segment-4-distance sin-tail-angle))))
;;     (draw-possibly-nuanced-circle tail-segment-4-center-x tail-segment-4-center-y (/ radius 6) color)
;;     (draw-possibly-nuanced-circle tail-segment-3-center-x tail-segment-3-center-y (/ radius 5) color)
;;     (draw-possibly-nuanced-circle tail-segment-2-center-x tail-segment-2-center-y (/ radius 4) color)
;;     (draw-possibly-nuanced-circle tail-segment-1-center-x tail-segment-1-center-y (/ radius 3) color)
;;     (draw-possibly-nuanced-circle left-eye-center-x left-eye-center-y eye-radius eye-color)
;;     (draw-possibly-nuanced-circle right-eye-center-x right-eye-center-y eye-radius eye-color)
;;     (draw-possibly-nuanced-circle x y radius color)
;;     (if name
;;         (draw-centered-text name x (+ y radius (font-height)) avatar-name-color))))

;;; Draw the player:
(define left-eye-delta-angle 0.5)
(define right-eye-delta-angle (- left-eye-delta-angle))
(define eye-color (light (light yellow)))
(define avatar-name-color (make-color 1 1 1))
(define tail-segment-no 3)
(define (draw-player x y radius angle name color)
  (let* ((left-eye-angle (+ angle left-eye-delta-angle))
         (right-eye-angle (+ angle right-eye-delta-angle))
         (tail-angle (+ angle pi))
         (sin-tail-angle (sin tail-angle))
         (cos-tail-angle (cos tail-angle))
         (eye-radius (/ radius 3))
         (left-eye-center-x (+ x (* radius (cos left-eye-angle))))
         (left-eye-center-y (- y (* radius (sin left-eye-angle))))
         (right-eye-center-x (+ x (* radius (cos right-eye-angle))))
         (right-eye-center-y (- y (* radius (sin right-eye-angle)))))
    (down-for i tail-segment-no 1
      (let ((segment-radius (* radius (/ 2 (+ i 2))))
            (segment-center-distance (* radius (+ i 1) 0.4)))
        (draw-possibly-nuanced-circle (+ x (* segment-center-distance cos-tail-angle))
                                      (- y (* segment-center-distance sin-tail-angle))
                                      segment-radius
                                      color)))
    (draw-possibly-nuanced-circle left-eye-center-x left-eye-center-y eye-radius eye-color)
    (draw-possibly-nuanced-circle right-eye-center-x right-eye-center-y eye-radius eye-color)
    (draw-possibly-nuanced-circle x y radius color)
    (if name
        (draw-centered-text name x (+ y radius (font-height)) avatar-name-color))))
