;;; This file is part of guile-with-graphics, a graphic library for Guile

;;; Copyright (C) 2009  Luca Saiu

;;; This program is free software: you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation, either version 3 of the License, or
;;; (at your option) any later version.

;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.

;;; You should have received a copy of the GNU General Public License
;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(initialize-graphics "Drawing-pad demo" 800 600)

(load "../color.scm") ;; this must be loaded *after* graphics is initialized
(load "../utility.scm")
(load "../graphics.scm")
(load "../interactive.scm")

(define tick-length 20)

(define current-x 0)
(define current-y 0)
(define previous-x 0)
(define previous-y 0)
(define drawing #f)

(define color (light (light yellow)))

(define segments-to-draw
  '())

(define (handle-event event)
  (cond ((quit-event? event)
         (exit 0))
        ((equal? event '(mouse-button-down right))
         (clear black)
         (refresh))
        ((equal? event '(mouse-button-down left))
         (set! drawing #t))
        ((equal? event '(mouse-button-up left))
         (set! segments-to-draw '())
         (set! drawing #f))
        ((equal? (car event) 'mouse-motion)
         (set! previous-x current-x)
         (set! previous-y current-y)
         (set! current-x (cadr event))
         (set! current-y (caddr event))
         (if drawing
             (set! segments-to-draw
                   (cons (list previous-x previous-y current-x current-y)
                         segments-to-draw))))
        (else
         (format #t "Unhandled event: ~s\n" event))))

(define (draw-segments segments color)
  (if (null? segments)
      'done
      (let* ((first-segment (car segments))
             (x1 (car first-segment))
             (y1 (cadr first-segment))
             (x2 (caddr first-segment))
             (y2 (cadddr first-segment))
             (other-segments (cdr segments)))
        (draw-line x1 y1 x2 y2 color)
        (draw-segments other-segments color))))

(define (on-tick seconds-since-last-call)
  ;; Handle all enqueued events:
  (handle-all-enqueued-events handle-event)
  
  ;; Draw enqueued segments and refresh, if there is anything to
  ;; draw:
  (when drawing
    (draw-segments segments-to-draw color)
    (set! segments-to-draw '())
    (refresh)))

(display "=============================================\n")
(display "  Draw by dragging the mouse with the left\n")
(display "  button. Right button to clear the window.\n")
(display "=============================================\n")
(clear black)
(call-with-maximum-frequency 100
                             on-tick)
