;;; This file is part of guile-with-graphics, a graphic library for Guile

;;; Copyright (C) 2009  Luca Saiu

;;; This program is free software: you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation, either version 3 of the License, or
;;; (at your option) any later version.

;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.

;;; You should have received a copy of the GNU General Public License
;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(initialize-graphics "Interactive demo" 800 600)

(load "../color.scm") ;; this must be loaded *after* graphics is initialized
(load "../utility.scm")
(load "../graphics.scm")
(load "../interactive.scm")


(define block-width 80)
(define block-height 25)
(define block-color (dark red))
(define block-x (/ (window-width) 2))
(define block-y (/ (window-height) 2))
(define delta-x 0)
(define delta-y 0)
(define changing-color #f)
(define time-at-startup (time))
(define tick-length 30)
(define speed 100)

(define (handle-event event)
  (cond ((quit-event? event)
         (exit 0))
        ((equal? event '(key-down left))
         (set! delta-x (- speed)))
        ((equal? event '(key-up left))
         (set! delta-x 0))
        ((equal? event '(key-down right))
         (set! delta-x speed))
        ((equal? event '(key-up right))
         (set! delta-x 0))
        ((equal? event '(key-down down))
         (set! delta-y speed))
        ((equal? event '(key-up down))
         (set! delta-y 0))
        ((equal? event '(key-down up))
         (set! delta-y (- speed)))
        ((equal? event '(key-up up))
         (set! delta-y 0))
        ((equal? event '(key-up space))
         (set! changing-color #f))
        ((equal? event '(key-down space))
         (set! changing-color #t))
        (else
         (format #t "Unhandled event: ~s\n" event))))

(define (on-tick seconds-since-last-call)
  ;; Handle all enqueued events:
  (handle-all-enqueued-events handle-event)
  
  ;; Compute the new position:
  (set! block-x (+ block-x (* seconds-since-last-call delta-x)))
  (set! block-y (+ block-y (* seconds-since-last-call delta-y)))
  
  ;; Change the color if needed;
  (if changing-color
      (set! block-color (random-color)))
  
  ;; Re-draw:
  (clear black)
  (draw-block block-x block-y block-width block-height block-color)
  (refresh))

(display "=============================================\n")
(display "  Use the arrow keys to move the block, and\n")
(display "  the space key to change its color.\n")
(display "=============================================\n")
(clear black)
(refresh)
(call-with-maximum-frequency 100
                             on-tick)
