;;; This file is part of guile-with-graphics, a graphic library for Guile

;;; Copyright (C) 2009  Luca Saiu

;;; This program is free software: you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation, either version 3 of the License, or
;;; (at your option) any later version.

;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.

;;; You should have received a copy of the GNU General Public License
;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


;;; Let's make the console functionality always available:
(load "console.scm")


(define (quit-event? event)
  (or (equal? event '(quit))
      ;;(equal? event '(key-down #\q))
      (equal? event '(key-down escape))))

(define (quit-event-handler event)
  (if (quit-event? event)
      (exit)
      (format #t "quit-event-handler: received the event ~s\n" event)))

;;; This is useful for simple tests, not for interactive-game; notice that
;;; this procedure handles all pending events (not just one) and it has
;;; zero parameters; hence it's *not* suitable to be used as a parameter
;;; of handle-all-enqueued-events:
(define (handle-quit-event-only)
  (handle-all-enqueued-events quit-event-handler))


;;; Handle all enqueued events with the given handler. Return #t if there
;;; was some event #f if there was none.
(define (handle-all-enqueued-events handle-one-event)
  (handle-all-enqueued-events-with-accumulator handle-one-event #f))
(define (handle-all-enqueued-events-with-accumulator handle-one-event did-we-see-some-event)
  (let ((next-event-or-#f (get-event)))
    (if next-event-or-#f
        (begin
          (handle-one-event next-event-or-#f)
          (handle-all-enqueued-events-with-accumulator handle-one-event #t))
        did-we-see-some-event)))

;;; This is useful as the last function call of test programs which are not loops:
(define (wait-till-the-user-quits)
  ;; Redraw the graphic window:
  (refresh)
  ;; Exit the program if the user has asked for it:
  (handle-quit-event-only)
  ;; Sleep for a short time, so that we don't use 100% of the processor time:
  (usleep 50)
  ;; Go again, in an infinite loop:
  (wait-till-the-user-quits))
